@@ -10,7 +10,8 @@ from time import time
 from datetime import datetime
 from io import open
 from collections import namedtuple
-from sqlite3 import OperationalError
+from sqlite3 import OperationalError, sqlite_version
+import shutil
 
 from cli_helpers.tabular_output import TabularOutputFormatter
 from cli_helpers.tabular_output import preprocessors
@@ -19,9 +20,9 @@ import sqlparse
 from prompt_toolkit.completion import DynamicCompleter
 from prompt_toolkit.enums import DEFAULT_BUFFER, EditingMode
 from prompt_toolkit.shortcuts import PromptSession, CompleteStyle
-from prompt_toolkit.styles.pygments import style_from_pygments_cls
 from prompt_toolkit.document import Document
 from prompt_toolkit.filters import HasFocus, IsDone
+from prompt_toolkit.formatted_text import ANSI
 from prompt_toolkit.layout.processors import (
     HighlightMatchingBracketProcessor,
     ConditionalProcessor,
@@ -41,7 +42,6 @@ from .clibuffer import cli_is_multiline
 from .completion_refresher import CompletionRefresher
 from .config import config_location, ensure_dir_exists, get_config
 from .key_bindings import cli_bindings
-from .encodingutils import utf8tounicode, text_type
 from .lexer import LiteCliLexer
 from .__init__ import __version__
 from .packages.filepaths import dir_path_exists
@@ -57,7 +57,6 @@ PACKAGE_ROOT = os.path.abspath(os.path.dirname(__file__))
 
 
 class LiteCli(object):
-
     default_prompt = "\\d> "
     max_len_prompt = 45
 
@@ -83,17 +82,17 @@ class LiteCli(object):
         self.formatter.litecli = self
         self.syntax_style = c["main"]["syntax_style"]
         self.less_chatty = c["main"].as_bool("less_chatty")
+        self.show_bottom_toolbar = c["main"].as_bool("show_bottom_toolbar")
         self.cli_style = c["colors"]
         self.output_style = style_factory_output(self.syntax_style, self.cli_style)
         self.wider_completion_menu = c["main"].as_bool("wider_completion_menu")
+        self.autocompletion = c["main"].as_bool("autocompletion")
         c_dest_warning = c["main"].as_bool("destructive_warning")
         self.destructive_warning = c_dest_warning if warn is None else warn
         self.login_path_as_host = c["main"].as_bool("login_path_as_host")
 
         # read from cli argument or user config file
-        self.auto_vertical_output = auto_vertical_output or c["main"].as_bool(
-            "auto_vertical_output"
-        )
+        self.auto_vertical_output = auto_vertical_output or c["main"].as_bool("auto_vertical_output")
 
         # audit log
         if self.logfile is None and "audit_log" in c["main"]:
@@ -106,6 +105,11 @@ class LiteCli(object):
                     fg="red",
                 )
                 self.logfile = False
+        # Load startup commands.
+        try:
+            self.startup_commands = c["startup_commands"]
+        except KeyError:  # Redundant given the load_config() function that merges in the standard config, but put here to avoid fail if user do not have updated config file.
+            self.startup_commands = None
 
         self.completion_refresher = CompletionRefresher()
 
@@ -113,9 +117,7 @@ class LiteCli(object):
         self.initialize_logging()
 
         prompt_cnf = self.read_my_cnf_files(["prompt"])["prompt"]
-        self.prompt_format = (
-            prompt or prompt_cnf or c["main"]["prompt"] or self.default_prompt
-        )
+        self.prompt_format = prompt or prompt_cnf or c["main"]["prompt"] or self.default_prompt
         self.prompt_continuation_format = c["main"]["prompt_continuation"]
         keyword_casing = c["main"].get("keyword_casing", "auto")
 
@@ -159,10 +161,11 @@ class LiteCli(object):
         )
         special.register_special_command(
             self.execute_from_file,
-            "source",
+            ".read",
             "\\. filename",
             "Execute commands from file.",
-            aliases=("\\.",),
+            case_sensitive=True,
+            aliases=("\\.", "source"),
         )
         special.register_special_command(
             self.change_prompt_format,
@@ -225,11 +228,14 @@ class LiteCli(object):
         return [(None, None, None, "Changed prompt format to %s" % arg)]
 
     def initialize_logging(self):
-
         log_file = self.config["main"]["log_file"]
         if log_file == "default":
             log_file = config_location() + "log"
-        ensure_dir_exists(log_file)
+        try:
+            ensure_dir_exists(log_file)
+        except OSError:
+            # Unable to create log file, log to temp directory instead.
+            log_file = "/tmp/litecli.log"
 
         log_level = self.config["main"]["log_level"]
 
@@ -256,10 +262,7 @@ class LiteCli(object):
             )
             return
 
-        formatter = logging.Formatter(
-            "%(asctime)s (%(process)d/%(threadName)s) "
-            "%(name)s %(levelname)s - %(message)s"
-        )
+        formatter = logging.Formatter("%(asctime)s (%(process)d/%(threadName)s) " "%(name)s %(levelname)s - %(message)s")
 
         handler.setFormatter(formatter)
 
@@ -293,7 +296,6 @@ class LiteCli(object):
         return {x: get(x) for x in keys}
 
     def connect(self, database=""):
-
         cnf = {"database": None}
 
         cnf = self.read_my_cnf_files(cnf.keys())
@@ -316,7 +318,7 @@ class LiteCli(object):
             exit(1)
 
     def handle_editor_command(self, text):
-        """Editor command is any query that is prefixed or suffixed by a '\e'.
+        R"""Editor command is any query that is prefixed or suffixed by a '\e'.
         The reason for a while loop is because a user might edit a query
         multiple times. For eg:
 
@@ -358,8 +360,7 @@ class LiteCli(object):
         else:
             history = None
             self.echo(
-                'Error: Unable to open the history file "{}". '
-                "Your query history will not be saved.".format(history_file),
+                'Error: Unable to open the history file "{}". ' "Your query history will not be saved.".format(history_file),
                 err=True,
                 fg="red",
             )
@@ -367,19 +368,15 @@ class LiteCli(object):
         key_bindings = cli_bindings(self)
 
         if not self.less_chatty:
-            print("Version:", __version__)
-            print("Mail: https://groups.google.com/forum/#!forum/litecli-users")
+            print(f"LiteCli: {__version__} (SQLite: {sqlite_version})")
             print("GitHub: https://github.com/dbcli/litecli")
-            # print("Home: https://litecli.com")
 
         def get_message():
             prompt = self.get_prompt(self.prompt_format)
-            if (
-                self.prompt_format == self.default_prompt
-                and len(prompt) > self.max_len_prompt
-            ):
+            if self.prompt_format == self.default_prompt and len(prompt) > self.max_len_prompt:
                 prompt = self.get_prompt("\\d> ")
-            return [("class:prompt", prompt)]
+            prompt = prompt.replace("\\x1b", "\x1b")
+            return ANSI(prompt)
 
         def get_continuation(width, line_number, is_soft_wrap):
             continuation = " " * (width - 1) + " "
@@ -457,9 +454,7 @@ class LiteCli(object):
                     else:
                         max_width = None
 
-                    formatted = self.format_output(
-                        title, cur, headers, special.is_expanded_output(), max_width
-                    )
+                    formatted = self.format_output(title, cur, headers, special.is_expanded_output(), max_width)
 
                     t = time() - start
                     try:
@@ -480,29 +475,17 @@ class LiteCli(object):
             except EOFError as e:
                 raise e
             except KeyboardInterrupt:
-                # get last connection id
-                connection_id_to_kill = sqlexecute.connection_id
-                logger.debug("connection id to kill: %r", connection_id_to_kill)
-                # Restart connection to the database
-                sqlexecute.connect()
                 try:
-                    for title, cur, headers, status in sqlexecute.run(
-                        "kill %s" % connection_id_to_kill
-                    ):
-                        status_str = str(status).lower()
-                        if status_str.find("ok") > -1:
-                            logger.debug(
-                                "cancelled query, connection id: %r, sql: %r",
-                                connection_id_to_kill,
-                                text,
-                            )
-                            self.echo("cancelled query", err=True, fg="red")
+                    sqlexecute.conn.interrupt()
                 except Exception as e:
                     self.echo(
                         "Encountered error while cancelling query: {}".format(e),
                         err=True,
                         fg="red",
                     )
+                else:
+                    logger.debug("cancelled query")
+                    self.echo("cancelled query", err=True, fg="red")
             except NotImplementedError:
                 self.echo("Not Yet Implemented.", fg="yellow")
             except OperationalError as e:
@@ -529,10 +512,6 @@ class LiteCli(object):
                 logger.error("traceback: %r", traceback.format_exc())
                 self.echo(str(e), err=True, fg="red")
             else:
-                if is_dropping_database(text, self.sqlexecute.dbname):
-                    self.sqlexecute.dbname = None
-                    self.sqlexecute.connect()
-
                 # Refresh the table names and column names if necessary.
                 if need_completion_refresh(text):
                     self.refresh_completions(reset=need_completion_reset(text))
@@ -549,8 +528,10 @@ class LiteCli(object):
         else:
             complete_style = CompleteStyle.COLUMN
 
-        with self._completer_lock:
+        if not self.autocompletion:
+            complete_style = CompleteStyle.READLINE_LIKE
 
+        with self._completer_lock:
             if self.key_bindings == "vi":
                 editing_mode = EditingMode.VI
             else:
@@ -561,7 +542,7 @@ class LiteCli(object):
                 reserve_space_for_menu=self.get_reserved_space(),
                 message=get_message,
                 prompt_continuation=get_continuation,
-                bottom_toolbar=get_toolbar_tokens,
+                bottom_toolbar=get_toolbar_tokens if self.show_bottom_toolbar else None,
                 complete_style=complete_style,
                 input_processors=[
                     ConditionalProcessor(
@@ -585,6 +566,42 @@ class LiteCli(object):
                 search_ignore_case=True,
             )
 
+        def startup_commands():
+            if self.startup_commands:
+                if "commands" in self.startup_commands:
+                    if isinstance(self.startup_commands["commands"], str):
+                        commands = [self.startup_commands["commands"]]
+                    else:
+                        commands = self.startup_commands["commands"]
+                    for command in commands:
+                        try:
+                            res = sqlexecute.run(command)
+                        except Exception as e:
+                            click.echo(command)
+                            self.echo(str(e), err=True, fg="red")
+                        else:
+                            click.echo(command)
+                            for title, cur, headers, status in res:
+                                if title == "dot command not implemented":
+                                    self.echo(
+                                        "The SQLite dot command '" + command.split(" ", 1)[0] + "' is not yet implemented.",
+                                        fg="yellow",
+                                    )
+                                else:
+                                    output = self.format_output(title, cur, headers)
+                                    for line in output:
+                                        self.echo(line)
+                else:
+                    self.echo(
+                        "Could not read commands. The startup commands needs to be formatted as: \n commands = 'command1', 'command2', ...",
+                        fg="yellow",
+                    )
+
+        try:
+            startup_commands()
+        except Exception as e:
+            self.echo("Could not execute all startup commands: \n" + str(e), fg="yellow")
+
         try:
             while True:
                 one_iteration()
@@ -597,7 +614,7 @@ class LiteCli(object):
     def log_output(self, output):
         """Log the output in the audit log, if it's enabled."""
         if self.logfile:
-            click.echo(utf8tounicode(output), file=self.logfile)
+            click.echo(output, file=self.logfile)
 
     def echo(self, s, **kwargs):
         """Print a message to stdout.
@@ -613,11 +630,7 @@ class LiteCli(object):
     def get_output_margin(self, status=None):
         """Get the output margin (number of rows for the prompt, footer and
         timing message."""
-        margin = (
-            self.get_reserved_space()
-            + self.get_prompt(self.prompt_format).count("\n")
-            + 2
-        )
+        margin = self.get_reserved_space() + self.get_prompt(self.prompt_format).count("\n") + 2
         if status:
             margin += 1 + status.count("\n")
 
@@ -703,13 +716,10 @@ class LiteCli(object):
             },
         )
 
-        return [
-            (None, None, None, "Auto-completion refresh started in the background.")
-        ]
+        return [(None, None, None, "Auto-completion refresh started in the background.")]
 
     def _on_completions_refreshed(self, new_completer):
-        """Swap the completer object in cli with the newly created completer.
-        """
+        """Swap the completer object in cli with the newly created completer."""
         with self._completer_lock:
             self.completer = new_completer
 
@@ -720,15 +730,14 @@ class LiteCli(object):
 
     def get_completions(self, text, cursor_positition):
         with self._completer_lock:
-            return self.completer.get_completions(
-                Document(text=text, cursor_position=cursor_positition), None
-            )
+            return self.completer.get_completions(Document(text=text, cursor_position=cursor_positition), None)
 
     def get_prompt(self, string):
         self.logger.debug("Getting prompt")
         sqlexecute = self.sqlexecute
         now = datetime.now()
         string = string.replace("\\d", sqlexecute.dbname or "(none)")
+        string = string.replace("\\f", os.path.basename(sqlexecute.dbname or "(none)"))
         string = string.replace("\\n", "\n")
         string = string.replace("\\D", now.strftime("%a %b %d %H:%M:%S %Y"))
         string = string.replace("\\m", now.strftime("%M"))
@@ -767,13 +776,7 @@ class LiteCli(object):
         if cur:
             column_types = None
             if hasattr(cur, "description"):
-
-                def get_col_type(col):
-                    # col_type = FIELD_TYPES.get(col[1], text_type)
-                    # return col_type if type(col_type) is type else text_type
-                    return text_type
-
-                column_types = [get_col_type(col) for col in cur.description]
+                column_types = [str(col) for col in cur.description]
 
             if max_width is not None:
                 cur = list(cur)
@@ -783,31 +786,25 @@ class LiteCli(object):
                 headers,
                 format_name="vertical" if expanded else None,
                 column_types=column_types,
-                **output_kwargs
+                **output_kwargs,
             )
 
-            if isinstance(formatted, (text_type)):
+            if isinstance(formatted, str):
                 formatted = formatted.splitlines()
             formatted = iter(formatted)
 
             first_line = next(formatted)
             formatted = itertools.chain([first_line], formatted)
 
-            if (
-                not expanded
-                and max_width
-                and headers
-                and cur
-                and len(first_line) > max_width
-            ):
+            if not expanded and max_width and headers and cur and len(first_line) > max_width:
                 formatted = self.formatter.format_output(
                     cur,
                     headers,
                     format_name="vertical",
                     column_types=column_types,
-                    **output_kwargs
+                    **output_kwargs,
                 )
-                if isinstance(formatted, (text_type)):
+                if isinstance(formatted, str):
                     formatted = iter(formatted.splitlines())
 
             output = itertools.chain(output, formatted)
@@ -818,7 +815,7 @@ class LiteCli(object):
         """Get the number of lines to reserve for the completion menu."""
         reserved_space_ratio = 0.45
         max_reserved_space = 8
-        _, height = click.get_terminal_size()
+        _, height = shutil.get_terminal_size()
         return min(int(round(height * reserved_space_ratio)), max_reserved_space)
 
     def get_last_query(self):
@@ -852,13 +849,9 @@ class LiteCli(object):
     is_flag=True,
     help="Automatically switch to vertical output mode if the result is wider than the terminal width.",
 )
-@click.option(
-    "-t", "--table", is_flag=True, help="Display batch output in table format."
-)
+@click.option("-t", "--table", is_flag=True, help="Display batch output in table format.")
 @click.option("--csv", is_flag=True, help="Display batch output in CSV format.")
-@click.option(
-    "--warn/--no-warn", default=None, help="Warn before running a destructive query."
-)
+@click.option("--warn/--no-warn", default=None, help="Warn before running a destructive query.")
 @click.option("-e", "--execute", type=str, help="Execute command and quit.")
 @click.argument("database", default="", nargs=1)
 def cli(
@@ -926,10 +919,7 @@ def cli(
         except (FileNotFoundError, OSError):
             litecli.logger.warning("Unable to open TTY as stdin.")
 
-        if (
-            litecli.destructive_warning
-            and confirm_destructive_query(stdin_text) is False
-        ):
+        if litecli.destructive_warning and confirm_destructive_query(stdin_text) is False:
             exit(0)
         try:
             new_line = True
@@ -966,31 +956,6 @@ def need_completion_refresh(queries):
             return False
 
 
-def is_dropping_database(queries, dbname):
-    """Determine if the query is dropping a specific database."""
-    if dbname is None:
-        return False
-
-    def normalize_db_name(db):
-        return db.lower().strip('`"')
-
-    dbname = normalize_db_name(dbname)
-
-    for query in sqlparse.parse(queries):
-        if query.get_name() is None:
-            continue
-
-        first_token = query.token_first(skip_cm=True)
-        _, second_token = query.token_next(0, skip_cm=True)
-        database_name = normalize_db_name(query.get_name())
-        if (
-            first_token.value.lower() == "drop"
-            and second_token.value.lower() in ("database", "schema")
-            and database_name == dbname
-        ):
-            return True
-
-
 def need_completion_reset(queries):
     """Determines if the statement is a database switch such as 'use' or '\\u'.
     When a database is changed the existing completions must be reset before we
