@@ -117,13 +117,15 @@ class SQLCompleter(Completer):
         "NOT",
         "NOTHING",
         "NULL",
+        "NULLS FIRST",
+        "NULLS LAST",
         "NUMERIC",
         "NVARCHAR",
         "OF",
         "OFFSET",
         "ON",
         "OR",
-        "ORDER",
+        "ORDER BY",
         "OUTER",
         "OVER",
         "PARTITION",
@@ -255,7 +257,7 @@ class SQLCompleter(Completer):
         self.reserved_words = set()
         for x in self.keywords:
             self.reserved_words.update(x.split())
-        self.name_pattern = compile("^[_a-z][_a-z0-9\$]*$")
+        self.name_pattern = compile(r"^[_a-z][_a-z0-9\$]*$")
 
         self.special_commands = []
         self.table_formats = supported_formats
@@ -265,11 +267,7 @@ class SQLCompleter(Completer):
         self.reset_completions()
 
     def escape_name(self, name):
-        if name and (
-            (not self.name_pattern.match(name))
-            or (name.upper() in self.reserved_words)
-            or (name.upper() in self.functions)
-        ):
+        if name and ((not self.name_pattern.match(name)) or (name.upper() in self.reserved_words) or (name.upper() in self.functions)):
             name = "`%s`" % name
 
         return name
@@ -435,10 +433,7 @@ class SQLCompleter(Completer):
                 return kw.upper()
             return kw.lower()
 
-        return (
-            Completion(z if casing is None else apply_case(z), -len(text))
-            for x, y, z in sorted(completions)
-        )
+        return (Completion(z if casing is None else apply_case(z), -len(text)) for x, y, z in sorted(completions))
 
     def get_completions(self, document, complete_event):
         word_before_cursor = document.get_word_before_cursor(WORD=True)
@@ -446,7 +441,6 @@ class SQLCompleter(Completer):
         suggestions = suggest_type(document.text, document.text_before_cursor)
 
         for suggestion in suggestions:
-
             _logger.debug("Suggestion type: %r", suggestion["type"])
 
             if suggestion["type"] == "column":
@@ -457,11 +451,7 @@ class SQLCompleter(Completer):
                     # drop_unique is used for 'tb11 JOIN tbl2 USING (...'
                     # which should suggest only columns that appear in more than
                     # one table
-                    scoped_cols = [
-                        col
-                        for (col, count) in Counter(scoped_cols).items()
-                        if count > 1 and col != "*"
-                    ]
+                    scoped_cols = [col for (col, count) in Counter(scoped_cols).items() if count > 1 and col != "*"]
 
                 cols = self.find_matches(word_before_cursor, scoped_cols)
                 completions.extend(cols)
@@ -534,9 +524,7 @@ class SQLCompleter(Completer):
                 )
                 completions.extend(queries)
             elif suggestion["type"] == "table_format":
-                formats = self.find_matches(
-                    word_before_cursor, self.table_formats, start_only=True, fuzzy=False
-                )
+                formats = self.find_matches(word_before_cursor, self.table_formats, start_only=True, fuzzy=False)
                 completions.extend(formats)
             elif suggestion["type"] == "file_name":
                 file_names = self.find_files(word_before_cursor)
